package PluS;

import java.io.File;
import java.io.FilenameFilter;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class PluginLoader {
	File[] xmlFiles;
	Map<String, Component> components;
	PluginLinker linkage;
	
	public PluginLoader(){
		xmlFiles = loadFiles();
		components = new HashMap<String, Component>();
		loadComponents();
		checkDependencies();
		linkage = new PluginLinker();
		linkage.link(Collections.unmodifiableMap(components));
	}
	
	private File[] loadFiles(){
		File pluginDir = new File("plugins");
		File[] result;
		if(pluginDir.exists()){
			result = pluginDir.listFiles(new FilenameFilter() {
				
				public boolean accept(File dir, String name) {
					 return name.substring(name.length()-4).equals(".xml");
				}
			
			});
		}
		else{
			result = new File[0];
		}
		return result;
	}
	
	private void loadComponents(){
		ComponentBuilder cBuilder = new ComponentBuilder();
		for(File f: xmlFiles){
			Component c = cBuilder.createComponent(f);
			if(c!=null)c.registerWith(this);
		}
	}
	
	private void checkDependencies(){
		boolean removed = false;
		do{
			Iterator<Component> sIter = components.values().iterator();
			while(sIter.hasNext()){
				Component c = sIter.next();
				if(!allPresent(c.getDependencies())){
					sIter.remove();
					removed = true;
				}
			}
		}while(removed);
	}
	
	/**
	 * Register a component with an ID
	 * @param ID
	 * @param comp
	 */
	public void register(String ID, Component comp){
		components.put(ID, comp);
	}
	
	/**
	 * Checks whether all the IDs in the given list are present in the existing components
	 * @param dependencyIDs
	 * @return
	 */
	public boolean allPresent(List<String> dependencyIDs){
		return components.keySet().containsAll(dependencyIDs);
	}
	
	/**
	 * Dynamically adds the given file to the java classpath
	 * @param jar
	 * @return
	 */
	public boolean addJarToClasspath(String jar){
		 
		try {
			Class[] parameters = new Class[]{URL.class};
			File f = new File(jar);
			URL u = f.toURL();	
			URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
			Class sysclass = URLClassLoader.class;
			Method method = sysclass.getDeclaredMethod("addURL",parameters);
			method.setAccessible(true);
			method.invoke(sysloader,new Object[]{ u });
		} catch (Throwable t) {
			t.printStackTrace();
			return false;
		}
		
		return true;
	}
	
	public static void main(String[] args){
		PluginLoader pluS = new PluginLoader();
	}
}
